//
//  Widget expanding/collapsing controller object
//

var sharedTimer = null;
var currentExpander = null;

function Expander( element, width_target, height_target ) {
    this.element = element;

    if ( element._expander_lock == undefined ) {
        element._expander_lock = false;
    }

    /* get initial position*/
    var style = document.defaultView.getComputedStyle( element );
    var left  = parseInt( style.getPropertyValue( 'left' ) );
    var top   = parseInt( style.getPropertyValue( 'top' ) );
    var w0    = parseInt( style.getPropertyValue( 'width' ) );
    var h0    = parseInt( style.getPropertyValue( 'height' ) );

    /* set closebox position */
    if ( window.widget ) {
        widget.setCloseBoxOffset( left + 10, top + 10 );
    }

    this.location = { x:left, y:top };
    this.size1 = { width:w0, height:h0 };
    this.size2 = { width:width_target, height:height_target };
    this.currentSize = { width:w0, height:h0 };
    this.transitionMode = true;

    this.counter           = 0;
    this.maxSteps          = 15;
    this.animationInterval = 33;

    // states and switches
    this.isMoving      = false;
    this.isExpanding   = false;
    this.isCollapsed   = true;
    this.positionFixed = false;

    // methods
    this.toggle   = _Expander__toggle;
    this.expand   = _Expander__expand;
    this.collapse = _Expander__collapse;
    this.canActivate = _Expander__canActivate;
    // private methods, do not call from other classes
    this.tick     = _Expander__tick; 
    //this.setCurrentSize = _Expander__setCurrentSize;

    // callback function afer motion
    this.callBack = null;
}

function _Expander__canActivate() {
    if ( this.isMoving ) return false;
    if ( this.element ) {
        if ( this.element._expander_lock == true ) return false;
        return true;
    }
    return false;
}

function __tick() {
    if ( currentExpander == null ) {
        clearInterval( sharedTimer );
        sharedTimer = null;
        return;
    }

    if ( currentExpander.tick() == false ) {
        clearInterval( sharedTimer );
        sharedTimer = null;
    }
}

function _Expander__tick() {
    this.counter ++;
    if ( this.counter > this.maxSteps ) {
        //sharedTimer = null;
        if ( this.isCollapsed ) {
            this.element.style.width  = this.size2.width + "px";
            this.element.style.height = this.size2.height + "px";
            this.isCollapsed = false;
        } else {
            this.element.style.width  = this.size1.width + "px";
            this.element.style.height = this.size1.height + "px";
            this.isCollapsed = true;
        }

        this.isMoving = false;
        this.element._expander_lock = false;

        currentExpander = null;
        if ( this.callBack != null ) {
            this.callBack();
        }
        //this.isCollapsed = !this.isCollapsed;
        return false;
    }
    
    var sr  = this.counter / ( this.maxSteps );
    var sr0 = ( this.counter - 1 ) / ( this.maxSteps );
    var pi  = 3.14159265;
    var ratio = Math.sin( sr * pi / 2 ) - Math.sin( sr0 * pi / 2 );

    var dx;
    var dy;
    if ( this.isExpanding ) {
        dx = this.size2.width  - this.size1.width;
        dy = this.size2.height - this.size1.height;
    } else {
        dx = - this.size2.width  + this.size1.width;
        dy = - this.size2.height + this.size1.height;
    }

    dx *= ratio;
    dy *= ratio;

    this.currentSize.width  += dx;
    this.currentSize.height += dy;

    //debug( "size = ( " + this.currentSize.width + ", " + this.currentSize.height + " ) : ( dx, dy ) = " + dx + ", " + dy);
    //debug( "size1 = ( " + this.size1.width + ", " + this.size1.height + " ), size2 = ( " + this.size2.width + ", " + this.size2.height + " ) " );

    this.element.style.width  = this.currentSize.width + "px";
    this.element.style.height = this.currentSize.height + "px";

    var dw = dx;
    var dh = dy;

    if ( this.transitionMode && this.positionFixed == false ) {
        var x_next; // horizontal position of next step
        var y_next; // vertical position of next step
        x_next = this.location.x - dx / 2;
        y_next = this.location.y - dy / 2;
        if ( x_next < 1 ) {
            x_next = 0;
        } else {
            dw /= 2;
        }
        if ( y_next < 1 ) {
            y_next = 0;
        } else {
            dh /= 2;
        }

//         this.element.style.left = parseInt( x_next ) + "px";
//         this.element.style.top  = parseInt( y_next ) + "px";
//         this.location = { x:x_next, y:y_next };
        this.element.style.left = x_next + "px";
        this.element.style.top  = y_next + "px";
        this.location.x = x_next;
        this.location.y = y_next;
//        this.location = { x:x_next, y:y_next };
//        debug( "Moving to " + x_next + ", " + y_next );
        if ( window.widget ) {
            widget.setCloseBoxOffset( x_next + 10, y_next + 10 );
        }
    }
    // positions of the widgets cannot be altered in Dashboard widget
    //window.moveBy( - dx / 2, - dy / 2 );

    var right = this.currentSize.width + this.location.x;
    if ( right < window.outerWidth ) {
        dw = 0;
    } else {
        dw = right - window.outerWidth;
    }

    var bottom = this.currentSize.height + this.location.y;
    if ( bottom < window.outerHeight ) {
        dh = 0;
    } else {
        dh = bottom - window.outerHeight;
    }

    window.resizeBy( dw , dh );
    
    return true;
}

function _Expander__toggle( event ) {
    if ( this.isCollapsed ) {
        this.expand();
    } else {
        this.collapse();
    }
}

function _Expander__expand( event ) {
    if ( this.isMoving || sharedTimer != null || this.element._expander_lock ) {
        debug( "cannot operate while moving" );
        return;
    }
    if ( this.isCollapsed == false ) {
        return;
    }
    if ( currentExpander ) {
        debug( "moving expander not cleared" );
        return;
    }

    this.currentSize.width  = parseInt( this.size1.width );
    this.currentSize.height = parseInt( this.size1.height );
    currentExpander = this;
    this.isMoving = true;
    this.element._expander_lock = true;
    this.isExpanding = true;

    this.counter = 0;
    sharedTimer = setInterval( __tick, this.animationInterval );

}

function _Expander__collapse( event ) {
    if ( this.isMoving || sharedTimer != null || this.element._expander_lock ) {
        debug( "cannot operate while moving" );
        return;
    }
    if ( this.isCollapsed ) {
        return;
    }
    if ( currentExpander ) {
        debug( "moving expander not cleared" );
        return;
    }

    this.currentSize.width  = parseInt( this.size2.width );
    this.currentSize.height = parseInt( this.size2.height );
    currentExpander = this;
    this.isMoving = true;
    this.element._expander_lock = true;
    this.isExpanding = false;

    this.counter = 0;
    sharedTimer = setInterval( __tick, this.animationInterval );

}
